@@ -4,11 +4,13 @@
 /**
  * Obviously replies on working DNS service
  */
-class CheckDomainTest extends \PHPUnit\Framework\TestCase {
-    public function testBasic() {
+class CheckDomainTest extends \PHPUnit\Framework\TestCase
+{
+    public function testBasic()
+    {
         $this->assertEquals('', check_domain('example.com'));
         $this->assertEquals('', check_domain('google.com'));
-        $this->assertRegExp('/ not discoverable in DNS/', check_domain('fishbeansblahblahblah' . uniqid() . '.com'));
+        $this->assertMatchesRegularExpression('/ not discoverable in DNS/', check_domain('fishbeansblahblahblah' . uniqid() . '.com'));
     }
 }
 
