@@ -1,7 +1,9 @@
 <?php
 
-class GeneratePasswordTest extends \PHPUnit\Framework\TestCase {
-    public function testBasic() {
+class GeneratePasswordTest extends \PHPUnit\Framework\TestCase
+{
+    public function testBasic()
+    {
         $one = generate_password();
 
         $two = generate_password();
@@ -12,7 +14,8 @@ class GeneratePasswordTest extends \PHPUnit\Framework\TestCase {
         $this->assertEquals(12, strlen($one));
     }
 
-    public function testLength() {
+    public function testLength()
+    {
         $one = generate_password(1);
 
         $ten = generate_password(10);
