@@ -1,7 +1,9 @@
 <?php
 
-class ListDomainsForAdminTest extends \PHPUnit\Framework\TestCase {
-    public function testBasic() {
+class ListDomainsForAdminTest extends \PHPUnit\Framework\TestCase
+{
+    public function testBasic()
+    {
         $this->assertEquals([], list_domains_for_admin('test@test.com'));
     }
 }
